"use strict";
/**
 * @license
 * Qwik Speak
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://github.com/robisim74/qwik-speak/blob/main/LICENSE
 */
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const qwik = require("@builder.io/qwik");
const build = require("@builder.io/qwik/build");
const SpeakContext = qwik.createContextId("qwikspeak");
const logWarn = (message) => {
  console.warn("\x1B[33mQwik Speak warn\x1B[0m %s", message);
};
const logDebug = (message) => {
  console.debug("\x1B[36mQwik Speak\x1B[0m %s", message);
};
const cache = {};
const memoize = (fn) => {
  return (...args) => {
    const stringArgs = JSON.stringify(args);
    return stringArgs in cache ? cache[stringArgs] : cache[stringArgs] = fn(...args);
  };
};
const loadTranslations = async (ctx, assets, runtimeAssets, langs) => {
  if (build.isDev || build.isServer || runtimeAssets) {
    const { locale, translation, translationFn, config } = ctx;
    if (build.isDev) {
      const conflictingAsset = assets?.find((asset) => runtimeAssets?.includes(asset)) || assets?.find((asset) => config.runtimeAssets?.includes(asset)) || runtimeAssets?.find((asset) => config.assets?.includes(asset));
      if (conflictingAsset)
        logWarn(`Speak component: conflict between assets and runtimeAssets '${conflictingAsset}'`);
    }
    let resolvedAssets;
    if (build.isDev || build.isServer)
      resolvedAssets = [
        ...assets ?? [],
        ...runtimeAssets ?? []
      ];
    else
      resolvedAssets = [
        ...runtimeAssets ?? []
      ];
    if (resolvedAssets.length === 0)
      return;
    const resolvedLangs = new Set(langs || []);
    resolvedLangs.add(locale.lang);
    for (const lang of resolvedLangs) {
      const memoized = memoize(translationFn.loadTranslation$);
      const tasks = resolvedAssets.map((asset) => memoized(lang, asset));
      const sources = await Promise.all(tasks);
      const assetSources = sources.map((source, i) => ({
        asset: resolvedAssets[i],
        source
      }));
      for (const data of assetSources)
        if (data?.source) {
          if (!build.isDev && build.isServer && assets?.includes(data.asset))
            for (let [key, value] of Object.entries(data.source)) {
              if (typeof value === "string")
                value = new String(value);
              translation[lang][key] = qwik.noSerialize(value);
            }
          else
            Object.assign(translation[lang], data.source);
        }
    }
  }
};
const getValue = (key, data, params, keySeparator = ".", keyValueSeparator = "@@") => {
  let defaultValue = void 0;
  [key, defaultValue] = separateKeyValue(key, keyValueSeparator);
  const value = key.split(keySeparator).reduce((acc, cur) => acc && acc[cur] !== void 0 ? acc[cur] : void 0, data);
  if (value) {
    if (typeof value === "string" || value instanceof String)
      return params ? transpileParams(value.toString(), params) : value.toString();
    if (typeof value === "object")
      return params ? JSON.parse(transpileParams(JSON.stringify(value), params)) : value;
  }
  if (defaultValue) {
    if (!/^[[{].*[\]}]$/.test(defaultValue) || /^{{/.test(defaultValue))
      return params ? transpileParams(defaultValue, params) : defaultValue;
    return params ? JSON.parse(transpileParams(defaultValue, params)) : JSON.parse(defaultValue);
  }
  return key;
};
const separateKeyValue = (key, keyValueSeparator) => {
  return key.split(keyValueSeparator);
};
const transpileParams = (value, params) => {
  const replacedValue = value.replace(/{{\s?([^{}\s]*)\s?}}/g, (substring, parsedKey) => {
    const replacer = params[parsedKey];
    return replacer !== void 0 ? replacer : substring;
  });
  return replacedValue;
};
const QwikSpeakProvider = /* @__PURE__ */ qwik.componentQrl(/* @__PURE__ */ qwik.inlinedQrl((props) => {
  const lang = qwik.useServerData("locale");
  const resolvedTranslationFn = {
    loadTranslation$: props.translationFn?.loadTranslation$ ?? /* @__PURE__ */ qwik.inlinedQrl(() => null, "QwikSpeakProvider_component_resolvedTranslationFn_MIDC22ueZrk")
  };
  let resolvedLocale = props.locale ?? props.config.supportedLocales.find((value) => value.lang === lang);
  if (!resolvedLocale) {
    resolvedLocale = props.config.defaultLocale;
    if (build.isDev)
      logWarn(`Locale not resolved. Fallback to default locale: ${props.config.defaultLocale.lang}`);
  } else if (build.isDev)
    logDebug(`Resolved locale: ${resolvedLocale.lang}`);
  const state = {
    locale: Object.assign({}, resolvedLocale),
    translation: Object.fromEntries(props.config.supportedLocales.map((value) => [
      value.lang,
      {}
    ])),
    config: {
      rewriteRoutes: props.config.rewriteRoutes,
      defaultLocale: props.config.defaultLocale,
      supportedLocales: props.config.supportedLocales,
      assets: props.config.assets,
      runtimeAssets: props.config.runtimeAssets,
      keySeparator: props.config.keySeparator || ".",
      keyValueSeparator: props.config.keyValueSeparator || "@@"
    },
    translationFn: resolvedTranslationFn
  };
  const { config } = state;
  qwik.useContextProvider(SpeakContext, state);
  qwik.useTaskQrl(/* @__PURE__ */ qwik.inlinedQrl(async () => {
    const [config2, props2, state2] = qwik.useLexicalScope();
    await loadTranslations(state2, config2.assets, config2.runtimeAssets, props2.langs);
  }, "QwikSpeakProvider_component_useTask_sfbNflx0Y2A", [
    config,
    props,
    state
  ]));
  return /* @__PURE__ */ qwik._jsxC(qwik.Slot, null, 3, "gB_0");
}, "QwikSpeakProvider_component_yigdOibvcXE"));
const useSpeakContext = () => qwik.useContext(SpeakContext);
const useSpeakLocale = () => qwik.useContext(SpeakContext).locale;
const useSpeakConfig = () => qwik.useContext(SpeakContext).config;
const Speak = /* @__PURE__ */ qwik.componentQrl(/* @__PURE__ */ qwik.inlinedQrl((props) => {
  const ctx = useSpeakContext();
  qwik.useTaskQrl(/* @__PURE__ */ qwik.inlinedQrl(async () => {
    const [ctx2, props2] = qwik.useLexicalScope();
    if (build.isDev) {
      if (!props2.assets && !props2.runtimeAssets)
        logWarn("Speak component: no assets provided");
      const duplicateAsset = ctx2.config.assets?.find((asset) => props2.assets?.includes(asset));
      if (duplicateAsset)
        logWarn(`Speak component: duplicate assets '${duplicateAsset}'`);
      const duplicateRuntimeAsset = ctx2.config.runtimeAssets?.find((asset) => props2.runtimeAssets?.includes(asset));
      if (duplicateRuntimeAsset)
        logWarn(`Speak component: duplicate runtimeAssets '${duplicateRuntimeAsset}'`);
    }
    await loadTranslations(ctx2, props2.assets, props2.runtimeAssets, props2.langs);
  }, "Speak_component_useTask_b8fpOLpx7NA", [
    ctx,
    props
  ]));
  return /* @__PURE__ */ qwik._jsxC(qwik.Slot, null, 3, "0m_0");
}, "Speak_component_AkJ7NeLjpWM"));
const inlineTranslate = (keys, ctx, params, lang) => {
  const { locale, translation, config } = ctx;
  lang ?? (lang = locale.lang);
  if (Array.isArray(keys))
    return keys.map((k) => getValue(k, translation[lang], params, config.keySeparator, config.keyValueSeparator));
  return getValue(keys, translation[lang], params, config.keySeparator, config.keyValueSeparator);
};
const useTranslate = () => {
  const ctx = useSpeakContext();
  const translate = (keys, params, lang) => {
    const { locale, translation, config } = ctx;
    lang ?? (lang = locale.lang);
    if (Array.isArray(keys))
      return keys.map((k) => getValue(k, translation[lang], params, config.keySeparator, config.keyValueSeparator));
    return getValue(keys, translation[lang], params, config.keySeparator, config.keyValueSeparator);
  };
  return qwik.noSerialize(translate);
};
const useTranslatePath = () => {
  const ctx = useSpeakContext();
  const normalizePath = (pathname) => {
    const { config } = ctx;
    const source = config.rewriteRoutes?.find((rewrite) => pathname === `/${rewrite.prefix}` || pathname.startsWith(`/${rewrite.prefix}/`) || pathname.startsWith(`${rewrite.prefix}/`));
    if (source) {
      pathname = pathname === `/${source.prefix}` ? "/" : pathname.startsWith("/") ? pathname.substring(`/${source.prefix}`.length) : pathname.substring(`${source.prefix}/`.length);
      const sourceEntries = Object.entries(source.paths).map(([from, to]) => [
        to,
        from
      ]);
      const revertedPaths = Object.fromEntries(sourceEntries);
      const splitted = pathname.split("/");
      const translated = splitted.map((part) => revertedPaths[part] ?? part);
      return translated.join("/");
    }
    return pathname;
  };
  const rewritePath = (pathname, prefix) => {
    const { config } = ctx;
    let splitted = pathname.split("/");
    const destination = config.rewriteRoutes?.find((rewrite) => rewrite.prefix === prefix);
    if (prefix && destination) {
      const keys = Object.keys(destination.paths);
      const translating = splitted.some((part) => keys.includes(part));
      if (pathname === "/" || translating)
        pathname = pathname.startsWith("/") ? `/${prefix}` + pathname : `${prefix}/` + pathname;
    }
    splitted = pathname.split("/");
    const translated = splitted.map((part) => destination?.paths[part] ?? part);
    return translated.join("/");
  };
  const slashPath = (pathname, rewrote) => {
    if (pathname.endsWith("/") && !rewrote.endsWith("/"))
      return rewrote + "/";
    else if (!pathname.endsWith("/") && rewrote.endsWith("/") && rewrote !== "/")
      return rewrote.substring(0, rewrote.length - 1);
    return rewrote;
  };
  const translateOne = (pathname, lang) => {
    const { locale } = ctx;
    lang ?? (lang = locale.lang);
    const normalized = normalizePath(pathname);
    const rewrote = rewritePath(normalized, lang);
    return slashPath(pathname, rewrote);
  };
  const translate = (pathname, lang) => {
    const { locale, config } = ctx;
    if (!config.rewriteRoutes) {
      if (build.isDev)
        logWarn(`SpeakConfig rewriteRoutes not found`);
      return pathname;
    }
    lang ?? (lang = locale.lang);
    if (Array.isArray(pathname))
      return pathname.map((path) => translateOne(path, lang));
    return translateOne(pathname, lang);
  };
  return qwik.noSerialize(translate);
};
const usePlural = () => {
  const ctx = useSpeakContext();
  const plural = (value, key, params, options, lang) => {
    const { locale, translation, config } = ctx;
    lang ?? (lang = locale.lang);
    value = +value;
    const rule = new Intl.PluralRules(lang, options).select(value);
    key = key ? `${key}${config.keySeparator}${rule}` : rule;
    return getValue(key, translation[lang], {
      value,
      ...params
    }, config.keySeparator, config.keyValueSeparator);
  };
  return qwik.noSerialize(plural);
};
const useFormatNumber = () => {
  const locale = useSpeakLocale();
  const formatNumber = (value, options, lang, currency) => {
    lang ?? (lang = locale.extension ?? locale.lang);
    currency = currency ?? locale.currency;
    value = +value;
    options = {
      ...options
    };
    if (currency)
      options.currency = currency;
    return new Intl.NumberFormat(lang, options).format(value);
  };
  return qwik.noSerialize(formatNumber);
};
const useFormatDate = () => {
  const locale = useSpeakLocale();
  const formateDate = (value, options, lang, timeZone) => {
    lang ?? (lang = locale.extension ?? locale.lang);
    timeZone = timeZone ?? locale.timeZone;
    value = new Date(value);
    options = {
      ...options
    };
    if (timeZone)
      options.timeZone = timeZone;
    return new Intl.DateTimeFormat(lang, options).format(value);
  };
  return qwik.noSerialize(formateDate);
};
const useRelativeTime = () => {
  const locale = useSpeakLocale();
  const relativeTime = (value, unit, options, lang) => {
    lang ?? (lang = locale.extension ?? locale.lang);
    value = +value;
    return new Intl.RelativeTimeFormat(lang, options).format(value, unit);
  };
  return qwik.noSerialize(relativeTime);
};
const useDisplayName = () => {
  const locale = useSpeakLocale();
  const displayName = (code, options, lang) => {
    lang ?? (lang = locale.extension ?? locale.lang);
    return new Intl.DisplayNames(lang, options).of(code) || code;
  };
  return qwik.noSerialize(displayName);
};
exports.QwikSpeakProvider = QwikSpeakProvider;
exports.Speak = Speak;
exports.inlineTranslate = inlineTranslate;
exports.useDisplayName = useDisplayName;
exports.useFormatDate = useFormatDate;
exports.useFormatNumber = useFormatNumber;
exports.usePlural = usePlural;
exports.useRelativeTime = useRelativeTime;
exports.useSpeakConfig = useSpeakConfig;
exports.useSpeakContext = useSpeakContext;
exports.useSpeakLocale = useSpeakLocale;
exports.useTranslate = useTranslate;
exports.useTranslatePath = useTranslatePath;
